@@ -1,4 +1,4 @@
-# Copyright 2022-2024 VMware, Inc.
+# Copyright 2022-2025 Broadcom.
 # SPDX-License-Identifier: Apache-2
 """
 This code is run when initializing the python interperter in a Relenv environment.
@@ -70,6 +70,17 @@ def relocate():
     return relocate.relocate
 
 
+def buildenv():
+    """
+    Late import relenv buildenv.
+    """
+    if not hasattr(buildenv, "builenv"):
+        buildenv.buildenv = path_import(
+            "relenv.buildenv", str(pathlib.Path(__file__).parent / "buildenv.py")
+        )
+    return buildenv.buildenv
+
+
 def get_major_version():
     """
     Current python major version.
@@ -137,7 +148,9 @@ def _build_shebang(func, *args, **kwargs):
             return func(self, *args, **kwargs)
         debug(f"Relenv - _build_shebang {scripts} {interpreter}")
         if sys.platform == "win32":
-            return str(pathlib.Path("#!<launcher_dir>") / interpreter).encode()
+            return (
+                str(pathlib.Path("#!<launcher_dir>") / interpreter).encode() + b"\r\n"
+            )
         return common().format_shebang("/" / interpreter).encode()
 
     return wrapped
@@ -323,6 +336,14 @@ def install_wheel_wrapper(func):
                 if relocate().is_elf(file):
                     debug(f"Relenv - Found elf {file}")
                     relocate().handle_elf(plat / file, rootdir / "lib", True, rootdir)
+                elif relocate().is_macho(file):
+                    otool_bin = shutil.which("otool")
+                    if otool_bin:
+                        relocate().handle_macho(str(plat / file), str(rootdir), True)
+                    else:
+                        debug(
+                            "The otool command is not available, please run `xcode-select --install`"
+                        )
 
     return wrapper
 
@@ -588,8 +609,11 @@ def wrap_pip_build_wheel(name):
     def wrap(func):
         @functools.wraps(func)
         def wrapper(*args, **kwargs):
-            dirs = common().work_dirs()
-            toolchain = dirs.toolchain / common().get_triplet()
+            if sys.platform != "linux":
+                return func(*args, **kwargs)
+            base_dir = common().DATA_DIR / "toolchain"
+            toolchain = base_dir / common().get_triplet()
+            cargo_home = str(common().DATA_DIR / "cargo")
             if not toolchain.exists():
                 debug("Unable to set CARGO_HOME no toolchain exists")
             else:
@@ -599,7 +623,6 @@ def wrap_pip_build_wheel(name):
                     f"-C link-arg=-L{relenvroot}/lib "
                     f"-C link-arg=-L{toolchain}/sysroot/lib"
                 )
-                cargo_home = str(toolchain / "cargo")
                 set_env_if_not_set("CARGO_HOME", cargo_home)
                 set_env_if_not_set("OPENSSL_DIR", relenvroot)
                 set_env_if_not_set("RUSTFLAGS", rustflags)
@@ -684,7 +707,12 @@ def wrap_locations(name):
 
         return wrapper
 
+    # get_scheme is not available on pip-19.2.3
+    # try:
     mod.get_scheme = wrap(mod.get_scheme)
+    # except AttributeError:
+    #    debug(f"Module {mod} does not have attribute get_scheme")
+
     return mod
 
 
@@ -812,13 +840,24 @@ def install_cargo_config():
     """
     if sys.platform != "linux":
         return
-    triplet = common().get_triplet()
+
+    # We need this as a late import for python < 3.12 becuase importing it will
+    # load the ssl module. Causing out setup_openssl method to fail to load
+    # fips module.
     dirs = common().work_dirs()
-    toolchain = dirs.toolchain / triplet
+    triplet = common().get_triplet()
+    cargo_home = dirs.data / "cargo"
+
+    toolchain = common().get_toolchain()
+    if not toolchain:
+        debug("Unable to set CARGO_HOME ppbt package not installed")
+        return
+
     if not toolchain.exists():
         debug("Unable to set CARGO_HOME no toolchain exists")
         return
-    cargo_home = toolchain / "cargo"
+
+    # cargo_home = dirs.data / "cargo"
     if not cargo_home.exists():
         cargo_home.mkdir()
     cargo_config = cargo_home / "config.toml"
@@ -843,12 +882,22 @@ def setup_openssl():
     """
     Configure openssl certificate locations.
     """
+    if sys.platform == "win32":
+        return
+
     openssl_bin = shutil.which("openssl")
     if not openssl_bin:
         debug("Could not find the 'openssl' binary in the path")
+        set_openssl_modules_dir(str(sys.RELENV / "lib" / "ossl-modules"))
+
+        if load_openssl_provider("default") == 0:
+            debug("Unable to load the default openssl provider")
+        if load_openssl_provider("legacy") == 0:
+            debug("Unable to load the legacy openssl provider")
+
         return
 
-    if "OPENSSL_MODULES" not in os.environ and sys.platform != "win32":
+    if "OPENSSL_MODULES" not in os.environ:
         # First try and load the system's fips provider. Then load relenv's
         # legacy and default providers. The fips provider must be loaded first
         # in order OpenSSl to work properly..
@@ -888,7 +937,7 @@ def setup_openssl():
     # Use system openssl dirs
     # XXX Should we also setup SSL_CERT_FILE, OPENSSL_CONF &
     # OPENSSL_CONF_INCLUDE?
-    if "SSL_CERT_DIR" not in os.environ and sys.platform != "win32":
+    if "SSL_CERT_DIR" not in os.environ:
         proc = subprocess.run(
             [openssl_bin, "version", "-d"],
             universal_newlines=True,
